import java.awt.event.*;
import java.util.*;

/**
 * Lab #5
 * CS 2334, Section SECTION NUMBER GOES HERE
 * DATE GOES HERE
 * <p>
 * Each object of this class represents a GUI enhanced model of a list of places.
 * The model of an MVC GUI.
 * </p>
 * @version 1.0
 *
 */
public class PlacesModel extends Places {

	/** List to keep track of who is registered to listen for events from the PlacesModel. */
	private ArrayList<ActionListener> actionListenerList;
	
	/**
	 * @param country 
	 */
	public PlacesModel(String country) {
		super(country);
	}

	/**
	 * @param country
	 * @param places
	 */
	public PlacesModel(String country, LinkedHashMap<String, Place> places) {
		super(country, places);
	}
	
	/**
	 * @param place
	 */
	public void addToPlaces(Place place) {
		super.addToPlaces(place);
		processEvent(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, "add place"));
	}
	
	public void clearPlaces() {
		super.clearPlaces();
		processEvent(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, "clear place list"));
	}
	
	// TODO: Override additional mutators from Places so that the view will know when the model is changed (see above for examples)

	
	/**
	 * Register an action event listener.
	 */
	public synchronized void addActionListener(ActionListener l) {
		if (actionListenerList == null)
			actionListenerList = new ArrayList<ActionListener>();
		actionListenerList.add(l);
	}
	
	/**
	 * Remove an action event listener.
	 */
	public synchronized void removeActionListener(ActionListener l) {
		if (actionListenerList != null && actionListenerList.contains(l))
			actionListenerList.remove(l);
	}
	
	/**
	 * Fire event.
	 */
	private void processEvent(ActionEvent e) {
		ArrayList<ActionListener> list;
		
		synchronized (this) {
			if (actionListenerList == null) return;
			list = (ArrayList<ActionListener>)actionListenerList.clone();
		}
		
		for (int i = 0; i < list.size(); i++) {
			ActionListener listener = list.get(i);
			listener.actionPerformed(e);
		}
	}
}